==> admin-bro-0.9.3.tgz: package/src/backend/controllers/api-controller.spec.js <==
const ApiController = require('./api-controller')
const Filter = require('../utils/filter')

describe('ApiController', function () {
  beforeEach(function () {
    this.total = 0
    this.fieldName = 'title'
    this.recordJSON = { title: 'recordTitle' }
    this.recordStub = {
      toJSON: () => this.recordJSON,
    }
    this.action = {
      name: 'actionName',
      handler: this.sinon.stub(),
    }
    const property = { name: () => this.fieldName, reference: () => false }
    this.resourceStub = {
      decorate: this.sinon.stub().returns({
        getListProperties: this.sinon.stub().returns([property]),
        titleProperty: () => ({ name: () => this.fieldName }),
        properties: { [property.name()]: property },
        resourceActions: () => [this.action],
        recordActions: () => [this.action],
        recordsDecorator: records => records,
      }),
      find: this.sinon.stub().returns([]),
      count: this.sinon.stub().returns(this.total),
      findOne: this.sinon.stub().returns(this.recordStub),
    }
    this.recordStub.resource = this.resourceStub
    this.adminStub = {
      findResource: this.sinon.stub().returns(this.resourceStub),
      options: { rootPath: '/admin' },
    }
    this.apiController = new ApiController({ admin: this.adminStub })

    this.sinon.stub(Filter.prototype, 'populate').returns([this.recordStub])
  })

  describe('#index', function () {
    beforeEach(async function () {
      this.response = await this.apiController.index({}, {})
    })

    it('returns default meta data when no were given', function () {
      expect(this.response.meta).to.deep.equal({
        total: this.total,
        perPage: 10,
        page: 1,
        direction: 'asc',
        sortBy: this.fieldName,
      })
    })

    it('returns an empty array of records', function () {
      expect(this.response.records).to.have.lengthOf(0)
    })
  })

  describe('#search', function () {
    it('returns records taken from the find method for given resource', async function () {
      const response = await this.apiController.search({ params: {} }, {})
      expect(response.records).to.have.lengthOf(0)
    })
  })

  describe('#get', function () {
    it('returns record', async function () {
      const response = await this.apiController.get({}, {})
      expect(response.record).to.deep.equal(this.recordJSON)
    })
  })

  describe('#resourceAction', function () {
    it('calls the handler of correct action', async function () {
      await this.apiController.resourceAction({
        params: {
          action: this.action.name,
        },
      }, {})
      expect(
        this.action.handler,
      ).to.have.been.calledWith(
        this.sinon.match.any,
        this.sinon.match.any,
        this.sinon.match.has('action', this.action),
      )
    })
  })

  describe('#recordAction', function () {
    it('calls the handler of correct action', async function () {
      await this.apiController.recordAction({
        params: {
          action: this.action.name,
        },
      }, {})
      expect(
        this.action.handler,
      ).to.have.been.calledWith(
        this.sinon.match.any,
        this.sinon.match.any,
        this.sinon.match.has('action', this.action).and(
          this.sinon.match.has('record', this.recordStub),
        ),
      )
    })
  })
})

==> admin-bro-1.1.2.tgz: package/src/backend/controllers/api-controller.spec.js <==
const ApiController = require('./api-controller')
const Filter = require('../utils/filter')

describe('ApiController', function () {
  beforeEach(function () {
    this.total = 0
    this.fieldName = 'title'
    this.recordJSON = { title: 'recordTitle' }
    this.recordStub = {
      toJSON: () => this.recordJSON,
    }
    this.resourceName = 'Users'
    this.action = {
      name: 'actionName',
      handler: this.sinon.stub(),
      isAccessible: this.sinon.stub().returns(true),
    }
    this.isActionAccessibleStub = this.sinon.stub()
    const property = { name: () => this.fieldName, reference: () => false }
    this.resourceStub = {
      decorate: this.sinon.stub().returns({
        actions: {
          list: this.action,
          edit: this.action,
          show: this.action,
          delete: this.action,
          new: this.action,
          [this.action.name]: {},
        },
        getListProperties: this.sinon.stub().returns([property]),
        titleProperty: () => ({ name: () => this.fieldName }),
        properties: { [property.name()]: property },
        resourceActions: () => [this.action],
        recordActions: () => [this.action],
        recordsDecorator: records => records,
        id: this.resourceName,
      }),
      find: this.sinon.stub().returns([]),
      count: this.sinon.stub().returns(this.total),
      findOne: this.sinon.stub().returns(this.recordStub),
    }
    this.recordStub.resource = this.resourceStub
    this.adminStub = {
      findResource: this.sinon.stub().returns(this.resourceStub),
      options: { rootPath: '/admin' },
    }
    this.currentAdmin = { email: 'john@doe.com', name: 'John' }
    this.apiController = new ApiController({ admin: this.adminStub }, this.currentAdmin)

    this.sinon.stub(Filter.prototype, 'populate').returns([this.recordStub])
  })

  describe('#search', function () {
    it('returns records taken from the find method for given resource', async function () {
      const response = await this.apiController.search({ params: {} }, {})
      expect(response.records).to.have.lengthOf(0)
    })

    it('throws an error when user doesn\'t have rights to access the list action', function (done) {
      this.action.isAccessible.returns(false)
      this.apiController.search({ params: {} }, {}).catch((error) => {
        expect(error.message).to.include('list')
        expect(error.name).to.equal('ForbiddenError')
        done()
      })
    })
  })

  describe('#resourceAction', function () {
    it('calls the handler of correct action', async function () {
      await this.apiController.resourceAction({
        params: {
          action: this.action.name,
        },
      }, {})
      expect(
        this.action.handler,
      ).to.have.been.calledWith(
        this.sinon.match.any,
        this.sinon.match.any,
        this.sinon.match.has('action', this.action),
      )
    })

    it('calls isActionAccessible on the resource decorator', async function () {
      await this.apiController.resourceAction({
        params: {
          action: this.action.name,
        },
      }, {})

      expect(this.action.isAccessible).to.have.been.calledWith(this.currentAdmin)
    })
  })

  describe('#recordAction', function () {
    it('calls the handler of correct action', async function () {
      await this.apiController.recordAction({
        params: {
          action: this.action.name,
          recordId: 'id',
        },
      }, {})
      expect(
        this.action.handler,
      ).to.have.been.calledWith(
        this.sinon.match.any,
        this.sinon.match.any,
        this.sinon.match.has('action', this.action).and(
          this.sinon.match.has('record', this.recordStub),
        ),
      )
    })

    it('calls isActionAccessible on the resource decorator', async function () {
      await this.apiController.recordAction({
        params: {
          action: this.action.name,
          recordId: 'id',
        },
      }, {})

      expect(this.action.isAccessible).to.have.been.calledWith(this.currentAdmin)
    })
  })
})
